# ----------------------------------------------------------------------------
# File-Commands
# ----------------------------------------------------------------------------

# Try to create a file if it doesn't exist and give it correct permissions

proc create_file {file_name} {
  global file_mod file_grp

  if [file exists $file_name] {return 0}
  if ![catch {open $file_name w} file_id] {
    catch {close $file_id}
    catch "eval exec chmod $file_mod $file_name"
    catch "eval exec chgrp $file_grp $file_name"
  }
}


# ----------------------------------------------------------------------------
# Read contents of a file

proc read_file {opts fileName} {

  if [catch {open "$fileName" r} file_id] {
    set result ""
  } else {
    if { $opts == "" } {
      set result [read $file_id]
    } else {
      set result [read $opts $file_id]
    }
    catch {close $file_id}
  }
  return "$result"
}


# ----------------------------------------------------------------------------
# Remove an entry of the current file

proc do_entry_remove {} {
  global curr_mode curr_dir curr_file file_list hint_list text plmwin

  set items [listbox_selection]
  if { $items == "" } {return 0}
  if ![file writable $curr_dir/$curr_file] \
     {log_entry "$curr_file: Permission denied" ; return 0}
  set active [listbox_active]
#.request before deleting
  set text "Delete selected entries?"
  set ret [interact label Delete text]
  set count 0
  if !$ret {
    log_entry "$curr_mode: Deleting selected entries"
    update idletasks
    foreach item $items {
      set number [expr $item - $count]
      $plmwin.display.listbox delete $number
      set file_list [del_listitem $file_list $number]
      set hint_list [del_listitem $hint_list $number]
      incr count
    }
    save_file
  }
  do_mmail_mod $curr_mode
  if $ret {listbox_restore $items $active}
}


# ----------------------------------------------------------------------------
# Rename an entry of the current file

proc do_entry_rename {} {
  global curr_mode curr_dir curr_file newname text plmwin
  global mgetty_log fax_db

  set items [listbox_selection]
  if { $items == "" } {return 0}
  if ![file writable $curr_dir/$curr_file] \
     {log_entry "Can't write to $curr_file" ; return 0}
  set active [listbox_active]
  set changes 0
  foreach line $items {
    set oldname [$plmwin.display.listbox get $line]
    set newname $oldname
    if { ![interact entry Rename newname] \
         && "$newname" != "" && "$newname" != "$oldname" } {
#.....update the listbox
      $plmwin.display.listbox delete $line
      $plmwin.display.listbox insert $line $newname
      listbox_restore $items $active
      set changes 1
    }
  }
  update idletasks
  if $changes save_file
  do_mmail_mod $curr_mode
  listbox_restore $items $active
}


# ----------------------------------------------------------------------------
# Clear current file

proc do_file_clear {} {
  global curr_mode curr_dir curr_file text tmp_dir plmwin

  if ![$plmwin.display.listbox size] {return 0}
  set items [listbox_selection]
  set active [listbox_active]
  set text "Clear file $curr_dir/$curr_file ?"
  set ret [interact label "Do it" text]
  if !$ret {
    if ![catch "eval exec cp -V numbered -b $curr_dir/$curr_file $tmp_dir >/dev/null"] {
      if [catch {open $curr_dir/$curr_file w} file_id] \
         {log_entry "$curr_file: Access denied" ; return 0}
      log_entry "Clear $curr_file - backup made in $tmp_dir"
      catch {close $file_id}
    }
  }
  do_mmail_mod $curr_mode
  if $ret {listbox_restore $items $active}
}


# ----------------------------------------------------------------------------
# Save current contents of the listbox into the current file

proc save_file {} {
  global curr_mode curr_dir curr_file file_list hint_list plmwin

  if [catch {open $curr_dir/$curr_file w} file_id] \
     {log_entry "$curr_file: Permission denied" ; return 0}
  $plmwin.display.listbox select set 0 end
  foreach count [listbox_selection] {
    set line [$plmwin.display.listbox get $count]
    if [string compare $line ""] {
      if ![string compare $curr_mode "Ring"] {
        set hint [lindex $hint_list $count]
        if [string compare $hint ""] {
          puts $file_id "$line ${hint}file=[lindex $file_list $count]"
        } else {
          puts $file_id $line
        }
      } else {
        puts $file_id $line
      }
    }
  }
  catch {close $file_id}
}


# ----------------------------------------------------------------------------
# Stop recording / playing and exit palmail

proc do_exit {} {
  exec_stop 2
  exit
}


# ----------------------------------------------------------------------------
# Remove marked files

proc do_remove {} {
  global curr_mode curr_dir file_list hint_list text index_db
  global ringlog_dir ringlog_name

  set items0 [lindex [listbox_selection] 0]
  set itemsend [lindex [listbox_selection] \
      [expr [llength [listbox_selection]] -1]]
  if {$items0 == ""} {return 0}
  set items [listbox_selection]
  if ![file writable $curr_dir] \
     {log_entry "Can't write to $curr_dir" ; return 0}
  set active [listbox_active]
  if { "$curr_mode" == "VocOUT" } {
    set n1 [lindex $file_list $items0]
    set n2 [lindex $file_list $itemsend]
  } else {
    set n1 [expr $items0 + 1]
    set n2 [expr $itemsend + 1]
  }
  if { $items0 == $itemsend } {set text "Delete selected file $n1 ?"} \
  else {set text "Delete selected files: $n1 .. $n2 ?"}
  set ret [interact label Delete text]
  if !$ret {
    foreach line $items {
      set n [lindex $file_list $line]
      set files [glob -nocomplain "$curr_dir/$n"]
      if { [llength $files] == 1 } {
        log_entry "[file tail $files]: Removing file"
      } else {
        log_entry "$n: Removing files"
      }
      catch "eval exec rm -f $files >/dev/null"
      if {    $curr_mode == "VocOUT" \
           && [lindex "$hint_list" "$line"] == "*" } {
#.......update the Index-file
        index_delete "$n"
        write_Index
      } elseif { "$curr_mode" == "VocIN" } {
#.......update the ringlog file
        set ringlog "$ringlog_dir/$ringlog_name"
        update_ringlog
        set ringlog_lines {}
        if { ![catch {open $ringlog r} file_id] } {
          while {![eof $file_id]} {
            gets $file_id rline
            set result "$rline"
            set index [string first " vocfile=$n" "$rline"]
            if { -1 != $index } {
              set compare [string range "$rline" $index end]
              if { "$compare" == " vocfile=$n" } {
#...............remove the entry
                set rline1 [string range "$rline" 0 $index]
                set result "${rline1}(removed)"
              }
            }
            lappend ringlog_lines "$result"
          }
          catch {close $file_id}
#.........write the new ringlog
          if { ![catch {open $ringlog w} file_id] } {
            foreach rline $ringlog_lines {
              if { "$rline" != "" } {
                puts $file_id "$rline"
              }
            }
            catch {close $file_id}
          }
        }
      }
    }
  }
  do_mmail_mod $curr_mode
  if $ret {listbox_restore $items $active}
}


# ----------------------------------------------------------------------------
# Rename marked files

proc do_rename {} {
  global curr_mode curr_dir file_list hint_list newname index_db text plmwin
  global ringlog_dir ringlog_name

  set items [listbox_selection]
  if { $items == "" } {return 0}
  if ![file writable $curr_dir] \
     {log_entry "Can't write to $curr_dir" ; return 0}
  set active [listbox_active]
  set changes 0
  foreach line $items {
    set line_name [lindex $file_list $line]
    set newname $line_name
    set oldname "$newname"
    if { ![interact entry Rename newname] && "$newname" != "" } {
      if { "$curr_mode" == "VocOUT" && "$newname" == "Index" } {
        log_entry "Rename: Can't use reserved name '$newname'"
      } else {
#.......replace spaces with underscores:
        regsub -all " " "$newname" "_" newname
        if { "$oldname" != "$newname" } {
#.........Check if there is already a file with this name
          set do_it 1
          set overwrite 0
          if [file exists "$curr_dir/$newname"] {
            set text "Overwrite existing file $newname ?"
            set do_it ![interact label OK text]
            if $do_it {set overwrite 1}
          }
          if $do_it {
            log_entry "$line_name: Renaming file to $newname"
            set command "mv -f \"$curr_dir/$oldname\" \"$curr_dir/$newname\""
            if [catch "eval exec $command"] {
              log_entry "$line_name: Renaming file -- failed"
            } else {
              set changes 1
              if { $curr_mode == "VocOUT" } {
#...............update the Index-file
                if $overwrite {index_delete "$newname"}
                if { [lindex $hint_list $line] == "*" } {
                  index_delete "$oldname"
                  lappend index_db "$newname"
                }
                write_Index
              } elseif { "$curr_mode" == "VocIN" } {
#...............update the ringlog file
                set ringlog "$ringlog_dir/$ringlog_name"
                update_ringlog
                set ringlog_lines {}
                if { ![catch {open $ringlog r} file_id] } {
                  while {![eof $file_id]} {
                    gets $file_id rline
                    set result "$rline"
                    set line_done 0
                    if $overwrite {
#.....................remove the old overwritten entry for the new name
                      set index [string first " vocfile=$newname" "$rline"]
                      if { -1 != $index } {
                        set compare [string range "$rline" $index end]
                        if { "$compare" == " vocfile=$newname" } {
                          set rline1 [string range "$rline" 0 $index]
                          set result "${rline1}(removed)"
                          set line_done 1
                        }
		      }
                    }
                    if { !$line_done } {
#.....................update the entry for the old name
                      set index [string first " vocfile=$oldname" "$rline"]
                      if { -1 != $index } {
                        set compare [string range "$rline" $index end]
                        if { "$compare" == " vocfile=$oldname" } {
                          regsub "vocfile=$oldname" "$rline" \
                                 "vocfile=$newname" rline2
                          regsub " ] $oldname vocfile=" "$rline2" \
                                 " ] $newname vocfile=" rline3
                          regsub " ] vocfile=" "$rline3" \
                                 " ] $newname vocfile=" result
                        }
                      }
                    }
                    lappend ringlog_lines "$result"
		  }
#.................write the new ringlog
                  catch {close $file_id}
                  if { ![catch {open $ringlog w} file_id] } {
                    foreach rline $ringlog_lines {
                      if { "$rline" != "" } {
                        puts $file_id "$rline"
                      }
                    }
                    catch {close $file_id}
                  }
                }
              }
            }
          }
        }
#.......update the listbox
        $plmwin.button.update invoke
        if !$changes {listbox_restore $items $active}
      }
    }
  }
  listbox_select_clear
  do_mmail_mod $curr_mode
  if !$changes {listbox_restore $items $active}
}

